/*
  Ŀ
    Application: Example of a tagging browse object                         
      File Name: TAGBROW.PRG                                                
         Author: Nicholas Solomon                                           
        Tweeker: Kevin S. Gallagher                                         
  Ĵ
   KSG 12/95 This copy of tagbrow.prg is 5.3 Clipper or higher usable. Also 
   added protection if no index is currently being used.                    
                                                                            
  
*/

#include "inkey.ch"
#include "tagbrow.ch"
#include "tbrowse.ch"       // 5.3 Clipper

/*
  Ŀ
       Function: TAGBROW()                                                  
  Ĵ
    Description: allow user a taggable browse pick object                   
         Params: cFieldName   =  Name of field used for selection           
                   cColName   =  Column header for above col. object        
                       nTop   =  top row of object                          
                      nLeft   =  left col of object                         
                   lMessage   =  .t. = include message (default to .t.)     
                     cTitle   =  title a box around the object              
         Return: (aSelected) = record numbers of selected items             
  
*/
function tagbrow( cFieldName, cColName, nTop, nLeft, lMessage, cTitle)
    local c
    local nKey
    local aSelected    := {}
    local exit_request := .f.
    local skey         := ""
    local cOldColor    := setcolor()
    local cScreen      := savescreen(0,0,maxrow(),maxcol())
    local bField       := fieldblock( cFieldName )
    local b
    local nFound
    local nLenField

    DEFAULT cColName TO cFieldName
    DEFAULT lMessage TO .t.

    /* get length of browse field to determine width of browse screen */
    nLenField := len(eval(bField))

    /* build a box if cTitle was passed - could be done better! */
    if cTitle != NIL
        TagTitle(nTop,nLeft,nLenField,cTitle)
    endif

    b:= TBrowseDB(nTop,nLeft,maxrow() -4,nLeft + nLenField)
    b:setkey(K_F1,{|| kg_help()})
    b:headsep   := ""
    b:footsep   := ""
    b:colorspec := "W/B,N/W,R+/B,R+/W,W+/B,W/N,N/W"
    /*
    * the 1st column MUST be the array for selected (tagged) record numbers
    */
    b:addColumn( TBColumnNew(,;
                    {|| if(ascan(aSelected,recno()) !=0,CHECKMARK,' ');
                    };
                 );
    )

    // block to highlight selected vs. unselected items
    b:addColumn(TBColumnNew( cColName, bField ))

    b:getcolumn(1):width      := 1
    b:getcolumn(1):defColor   := {3,3}
    b:getcolumn(1):colorBlock := {|| {5,5}}
    b:getcolumn(2):defColor   := {3,3}
    b:getcolumn(2):colorBlock := ;
        {|| if(ascan(aSelected,recno()) != 0,{3,4},{1,2})}

    // freeze the first column - don't allow them to go into it
    b:freeze := 1

    DISPBEGIN()
        while !b:stabilize() ; enddo
    DISPEND()

    while !exit_request
        // keep 'em out of first column
        if b:colPos <= b:freeze
            b:colPos := b:freeze + 1
        endif
        while nextkey() = 0 .and. !b:stabilize() ; enddo
        if (b:applykey(nKey := inkey(0)) = TBR_EXIT )
            exit_request = .T.
            exit                                        /* NOTE */
        endif

        do case
            // deletes ALL selections
            case nKey == K_DEL
                aSelected:={}
                if lMessage
                    ShowBotMsg(aSelected)
                endif
                b:refreshall()

            // select-unselect element
            case nKey == K_ENTER
                if ( (nFound:=ascan(aSelected, recno() )) != 0 )
                    adel(aSelected, nFound)
                    asize(aSelected, (len(aSelected)-1))
                else
                    aadd(aSelected, recno() )
                endif
                /* simply inform user how many are selected */
                if lMessage
                    ShowBotMsg(aSelected)
                endif
                b:refreshcurrent()

            // jump if alpha key is pressed
            // Could use improvments!
            case ( nKey > 47 .and. nKey < 123) .or. ;
                    nKey = 32 .or. nKey = 39
                // KSG gaurd from runtime errors
                if indexord() <> 0
                    if ! dbseek( upper( chr( nKey)))
                        dbgotop()
                    endif
                    b:refreshall()
                endif

        endcase
    enddo
    setcolor( cOldColor )
    restscreen(0,0,maxrow(),maxcol(), cScreen)
return(aSelected)


/*
*   Routine: TagTitle()
*          : 
*   Purpose: Keep code clean in caller
*          : 
*  Comments: Void
*          : 
*/
static procedure TagTitle(nTop,nLeft,nLenField,cTitle)
    @ nTop-1, nLeft-1 to maxrow()-3, nLeft + nLenField+1 color 'W/B'
    @ nTop-1, (nLeft + (int((nLeft + nLenField) - (nLeft-1)) / 2)) - ;
              (int( len(cTitle)  / 2)) - 1 say '' + cTitle + '' ;
    color 'RB+/B'
return

/*
*   Routine: ShowBotMsg()
*          : 
*   Purpose: Keep code clean in caller
*          : 
*  Comments: Void
*          : 
*/
static procedure ShowBotMsg(aSelected)
    @ maxrow(),0 say padc('You have selected '           +;
                     alltrim(str(len(aSelected))) + " "  +;
                     iif(len(aSelected)=1,'item','items'),; 
                     maxcol())
return
